@@ -1,9 +1,10 @@
+#!C:\perl\bin\perl.exe 
 package Net::BitTorrent::DHT;
 {
     use Socket
         qw[/F_INET/ /_DGRAM/ /SOL_SO/ /SO_RE/ /pack_sockaddr_in/ /inet_/];
     use Digest::SHA qw[sha1];
-    use Scalar::Util qw[weaken];
+    use Scalar::Util qw[weaken refaddr];
     use Carp qw[carp];
 
     #
@@ -53,18 +54,18 @@ package Net::BitTorrent::DHT;
                     )
                 );
                 $self = bless \$node_id, $class;
-                $_client{$self} = $args->{q[Client]};
-                weaken $_client{$self};
+                $_client{refaddr $self} = $args->{q[Client]};
+                weaken $_client{refaddr $self};
                 my ($socket, $address, $port) = $self->_open_socket(
                     $args->{q[LocalAddr]}, $args->{q[LocalPort]},
                     $args->{q[ReuseAddr]},    # XXX - undocumented
                     $args->{q[ReusePort]}     # XXX - undocumented
                 );
-                $node_id{$self}       = $node_id;
-                $routing_table{$self} = {};
-                $backup_nodes{$self}  = q[];
-                $tid{$self}           = qq[\0] x 5;    # 26^5 before rollover
-                $_client{$self}->_schedule(
+                $node_id{refaddr $self}       = $node_id;
+                $routing_table{refaddr $self} = {};
+                $backup_nodes{refaddr $self}  = q[];
+                $tid{refaddr $self}           = qq[\0] x 5;    # 26^5 before rollover
+                $_client{refaddr $self}->_schedule(
                                           {Code   => sub { shift->_pulse() },
                                            Time   => time,
                                            Object => $self
@@ -77,26 +78,26 @@ package Net::BitTorrent::DHT;
         sub _open_socket {
             my ($self, $host, $port, $reuse_address, $reuse_port) = @_;
             {    # backup
-                $extra{$self}{q[LocalHost]} = $host if defined $host;
-                $extra{$self}{q[LocalPort]} = $port if defined $port;
-                $extra{$self}{q[ReuseAddr]} = $reuse_address
+                $extra{refaddr $self}{q[LocalHost]} = $host if defined $host;
+                $extra{refaddr $self}{q[LocalPort]} = $port if defined $port;
+                $extra{refaddr $self}{q[ReuseAddr]} = $reuse_address
                     if defined $reuse_address;
-                $extra{$self}{q[ReusePort]} = $reuse_port
+                $extra{refaddr $self}{q[ReusePort]} = $reuse_port
                     if defined $reuse_port;
             }
             {    # restore from backup
-                $host = $extra{$self}{q[LocalHost]}
+                $host = $extra{refaddr $self}{q[LocalHost]}
                     if not defined $host
-                        && defined $extra{$self}{q[LocalHost]};
-                $port = $extra{$self}{q[LocalPort]}
+                        && defined $extra{refaddr $self}{q[LocalHost]};
+                $port = $extra{refaddr $self}{q[LocalPort]}
                     if not defined $port
-                        && defined $extra{$self}{q[LocalPort]};
-                $reuse_address = $extra{$self}{q[ReuseAddr]}
+                        && defined $extra{refaddr $self}{q[LocalPort]};
+                $reuse_address = $extra{refaddr $self}{q[ReuseAddr]}
                     if not defined $reuse_address
-                        && defined $extra{$self}{q[ReuseAddr]};
-                $reuse_port = $extra{$self}{q[ReusePort]}
+                        && defined $extra{refaddr $self}{q[ReuseAddr]};
+                $reuse_port = $extra{refaddr $self}{q[ReusePort]}
                     if not defined $reuse_port
-                        && defined $extra{$self}{q[ReusePort]};
+                        && defined $extra{refaddr $self}{q[ReusePort]};
             }
 
             # perldoc perlipc
@@ -119,30 +120,30 @@ package Net::BitTorrent::DHT;
            #}
             bind($_socket, pack_sockaddr_in($port, inet_aton($host)))
                 or return;
-            $socket{$self} = $_socket;
+            $socket{refaddr $self} = $_socket;
             my ($_port, $packed_ip)
                 = unpack_sockaddr_in(getsockname($_socket));
             my $ip = gethostbyaddr($packed_ip, AF_INET);
 
-            #$_client{$self}->_remove_connection($self);
-            $_client{$self}->_add_connection($self, q[ro]) or return;
+            #$_client{refaddr $self}->_remove_connection($self);
+            $_client{refaddr $self}->_add_connection($self, q[ro]) or return;
             return ($_socket, inet_ntoa($packed_ip), $_port);
         }
 
         # Accessors | Private
         sub _client {
             return if defined $_[1];
-            return $_client{$_[0]};
+            return $_client{refaddr +$_[0]};
         }
 
         sub _routing_table {
             return if defined $_[1];
-            return $routing_table{$_[0]};
+            return $routing_table{refaddr +$_[0]};
         }
 
         sub _socket {
             return if defined $_[1];
-            return $socket{$_[0]};
+            return $socket{refaddr +$_[0]};
         }
 
         sub _queue_outgoing {    # no-op
@@ -152,68 +153,68 @@ package Net::BitTorrent::DHT;
 
         sub _port {
             return                if defined $_[1];
-            $_[0]->_open_socket() if not $socket{$_[0]};
-            return                if not $socket{$_[0]};
+            $_[0]->_open_socket() if not $socket{refaddr +$_[0]};
+            return                if not $socket{refaddr +$_[0]};
             my ($_port, undef)
-                = unpack_sockaddr_in(getsockname($socket{$_[0]}));
+                = unpack_sockaddr_in(getsockname($socket{refaddr +$_[0]}));
             return $_port;
         }
 
         sub _host {
             return                if defined $_[1];
-            $_[0]->_open_socket() if not $socket{$_[0]};
-            return                if not $socket{$_[0]};
+            $_[0]->_open_socket() if not $socket{refaddr +$_[0]};
+            return                if not $socket{refaddr +$_[0]};
             my (undef, $addr)
-                = unpack_sockaddr_in(getsockname($socket{$_[0]}));
+                = unpack_sockaddr_in(getsockname($socket{refaddr +$_[0]}));
             return inet_ntoa($addr);
         }
 
         # Accesors | Public
         sub node_id {
             return if defined $_[1];
-            return $node_id{$_[0]};
+            return $node_id{refaddr +$_[0]};
         }
 
         # Methods | Private
         sub _pulse {
             my ($self) = @_;
-            $self->_open_socket() if not defined $socket{$self};
+            $self->_open_socket() if not defined $socket{refaddr $self};
             {    # get rid of old outgoing packets...
                 my @expired_requests = grep {
-                    $outstanding_queries{$self}{$_}->{timestamp}
+                    $outstanding_queries{refaddr $self}{$_}->{timestamp}
                         < (time - (60 * 5))
-                } keys %{$outstanding_queries{$self}};
+                } keys %{$outstanding_queries{refaddr $self}};
                 while (my $packet = shift @expired_requests) {
-                    delete $outstanding_queries{$self}{$packet};
+                    delete $outstanding_queries{refaddr $self}{$packet};
                 }
             }
             {    # get rid of old, useless DHT nodes...
                 my @expired_nodes = grep {
-                    my $timestamp = $routing_table{$self}{$_}->_last_seen;
+                    my $timestamp = $routing_table{refaddr $self}{$_}->_last_seen;
                     ($timestamp < (time - (60 * 30)))
                         or ($timestamp < (time - (60 * 5))
-                         and (not defined $routing_table{$self}{$_}->node_id))
-                } keys %{$routing_table{$self}};
+                         and (not defined $routing_table{refaddr $self}{$_}->node_id))
+                } keys %{$routing_table{refaddr $self}};
                 for my $node (@expired_nodes) {
 
-                    #$backup_nodes{$self} .=
+                    #$backup_nodes{refaddr $self} .=
                     #    compact(sprintf(q[%s:%s],
-                    #                    $routing_table{$self}{$node}->host,
-                    #                    $routing_table{$self}{$node}->port)
+                    #                    $routing_table{refaddr $self}{$node}->host,
+                    #                    $routing_table{refaddr $self}{$node}->port)
                     #    );
-                    delete $routing_table{$self}{$node};
+                    delete $routing_table{refaddr $self}{$node};
                 }
             }
-            my @hosts = sort keys %{$routing_table{$self}};
+            my @hosts = sort keys %{$routing_table{refaddr $self}};
         NODE: for my $packed_host (@hosts) {
-                my $node = $routing_table{$self}{$packed_host};
+                my $node = $routing_table{refaddr $self}{$packed_host};
                 if (    (not defined $node->node_id)
                     and ($node->_last_seen < (time - (60 * 5))))
-                {   delete $routing_table{$self}{$packed_host};
+                {   delete $routing_table{refaddr $self}{$packed_host};
                     next NODE;
                 }
             }
-            $_client{$self}->_schedule({Code   => sub { shift->_pulse },
+            $_client{refaddr $self}->_schedule({Code   => sub { shift->_pulse },
                                         Time   => time + 30,
                                         Object => $self
                                        }
@@ -228,14 +229,14 @@ package Net::BitTorrent::DHT;
                 sort {
                     ($a->node_id ^ $target) cmp($b->node_id ^ $target)
                     } grep { defined $_->node_id }
-                    values %{$routing_table{$self}}
+                    values %{$routing_table{refaddr $self}}
             ]->[0 .. 8];
         }
 
         sub _send {
             my ($self, $args) = @_;
-            if (not defined $socket{$self}) { $self->_open_socket() }
-            if (not defined $socket{$self}) {
+            if (not defined $socket{refaddr $self}) { $self->_open_socket() }
+            if (not defined $socket{refaddr $self}) {
                 return;
             }
             if (not defined $args) {
@@ -248,11 +249,11 @@ package Net::BitTorrent::DHT;
                 return;
             }
             if (defined $args->{q[node]}->node_id
-                and $args->{q[node]}->node_id eq $node_id{$self})
+                and $args->{q[node]}->node_id eq $node_id{refaddr $self})
             {   carp q[Cannot send packet to ourself!];
                 return;
             }
-            if (not send($socket{$self}, $args->{q[packet]},
+            if (not send($socket{refaddr $self}, $args->{q[packet]},
                          0,              $args->{q[node]}->_packed_host)
                 )
             {   carp sprintf q[Cannot send %d bytes to %s: [%d] %s],
@@ -262,14 +263,14 @@ package Net::BitTorrent::DHT;
             }
             if (defined $args->{q[t]} and defined $args->{q[type]})
             {    # don't save replies
-                $outstanding_queries{$self}{$args->{q[t]}} = {
+                $outstanding_queries{refaddr $self}{$args->{q[t]}} = {
                                            size => length($args->{q[packet]}),
                                            timestamp => time,
                                            node      => $args->{q[node]},
                                            type      => $args->{q[type]},
                                            args      => $args
                 };
-                weaken $outstanding_queries{$self}{$args->{q[t]}}{q[args]}
+                weaken $outstanding_queries{refaddr $self}{$args->{q[t]}}{q[args]}
                     {q[node]};
             }
 
@@ -287,22 +288,22 @@ package Net::BitTorrent::DHT;
         sub _rw {
             my ($self, $read, $write) = @_;
             return if not $read;
-            $self->_open_socket() if not defined $socket{$self};
+            $self->_open_socket() if not defined $socket{refaddr $self};
             my ($actual_read) = 0;
-            my $packed_host = recv($socket{$self}, my ($_data), $read, 0);
+            my $packed_host = recv($socket{refaddr $self}, my ($_data), $read, 0);
             if (defined $packed_host) {
                 $actual_read = length $_data;
                 my ($packet, $leftover)
                     = bdecode($_data);    # TODO: if $leftover, loop
                 if ((defined $packet) and (ref $packet eq q[HASH])) {
-                    $routing_table{$self}{$packed_host} ||=
+                    $routing_table{refaddr $self}{$packed_host} ||=
                         Net::BitTorrent::DHT::Node->new(
                                               {dht         => $self,
                                                packed_host => $packed_host,
                                                node_id     => undef
                                               }
                         );
-                    my $node = $routing_table{$self}{$packed_host};
+                    my $node = $routing_table{refaddr $self}{$packed_host};
                     if (ref $packet ne q[HASH])
                     {    # An attempt to track down a strange bug...
                         require Data::Dumper;
@@ -357,11 +358,11 @@ package Net::BitTorrent::DHT;
                        #warn qq[DHT Reply:\n]
                        #        . Data::Dumper->Dump(
                        #        [$packet,
-                       #        $outstanding_queries{$self}{$packet->{q[t]}}],
+                       #        $outstanding_queries{refaddr $self}{$packet->{q[t]}}],
                        #                             [qw[packet for]]);
-                            if (defined $outstanding_queries{$self}
+                            if (defined $outstanding_queries{refaddr $self}
                                 {$packet->{q[t]}})
-                            {   my $type = $outstanding_queries{$self}
+                            {   my $type = $outstanding_queries{refaddr $self}
                                     {$packet->{q[t]}}{q[type]};
                                 if (defined $dispatch{$type}) {
                                     $dispatch{$type}($node, $packet);
@@ -371,13 +372,13 @@ package Net::BitTorrent::DHT;
                                     die qq[Unhandled DHT Reply:\n]
                                         . Data::Dumper->Dump(
                                                [$packet,
-                                                $outstanding_queries{$self}
+                                                $outstanding_queries{refaddr $self}
                                                     {$packet->{q[t]}}
                                                ],
                                                [qw[packet for]]
                                         );
                                 }
-                                delete $outstanding_queries{$self}
+                                delete $outstanding_queries{refaddr $self}
                                     {$packet->{q[t]}};
                             }
                         }
@@ -400,28 +401,28 @@ package Net::BitTorrent::DHT;
         sub _generate_token_id {    # automatic rollover/expansion/etc
             return if defined $_[1];
             my ($self) = @_;
-            my ($len) = ($tid{$self} =~ m[^([a-z]+)]);
-            $tid{$self} = (($tid{$self} =~ m[^z*(\0*)$])
-                           ? ($tid{$self} =~ m[\0]
-                              ? pack(q[a] . (length $tid{$self}),
+            my ($len) = ($tid{refaddr $self} =~ m[^([a-z]+)]);
+            $tid{refaddr $self} = (($tid{refaddr $self} =~ m[^z*(\0*)$])
+                           ? ($tid{refaddr $self} =~ m[\0]
+                              ? pack(q[a] . (length $tid{refaddr $self}),
                                      (q[a] x (length($len || q[]) + 1))
                                   )
-                              : (q[a] . (qq[\0] x (length($tid{$self}) - 1)))
+                              : (q[a] . (qq[\0] x (length($tid{refaddr $self}) - 1)))
                                )
-                           : ++$tid{$self}
+                           : ++$tid{refaddr $self}
             );
-            return $tid{$self};
+            return $tid{refaddr $self};
         }
 
         sub _add_node {
             my ($self, $node) = @_;
-            if (scalar(keys %{$routing_table{$self}}) < 300)
+            if (scalar(keys %{$routing_table{refaddr $self}}) < 300)
             {    # xxx - make this max variable
                 my ($host, $port) = split q[:], $node, 2;
                 my $packed_host = pack_sockaddr_in($port, inet_aton($host));
 
                 # We figure out what sort of node this is after the 1st packet
-                return $routing_table{$self}{$packed_host} ||=
+                return $routing_table{refaddr $self}{$packed_host} ||=
                     Net::BitTorrent::DHT::Node->new(
                                               {dht         => $self,
                                                packed_host => $packed_host,
@@ -429,7 +430,7 @@ package Net::BitTorrent::DHT;
                                               }
                     );
             }
-            return $backup_nodes{$self} .= compact($node);
+            return $backup_nodes{refaddr $self} .= compact($node);
         }
 
         sub _disconnect {    # noop
@@ -448,15 +449,15 @@ package Net::BitTorrent::DHT;
             my ($self) = @_;
 
             #
-            delete $socket{$self};
-            delete $fileno{$self};
-            delete $node_id{$self};
-            delete $_client{$self};
-            delete $tid{$self};
-            delete $routing_table{$self};
-            delete $backup_nodes{$self};
-            delete $outstanding_queries{$self};
-            delete $extra{$self};
+            delete $socket{refaddr $self};
+            delete $fileno{refaddr $self};
+            delete $node_id{refaddr $self};
+            delete $_client{refaddr $self};
+            delete $tid{refaddr $self};
+            delete $routing_table{refaddr $self};
+            delete $backup_nodes{refaddr $self};
+            delete $outstanding_queries{refaddr $self};
+            delete $extra{refaddr $self};
 
             #
             return 1;
