@@ -1,3 +1,4 @@
+#!C:\perl\bin\perl.exe 
 package Net::BitTorrent::Version;
 {
     use strict;
@@ -6,15 +7,17 @@ package Net::BitTorrent::Version;
     #
     use version qw[qv];    # core as of 5.009
     our $SVN = q[$Id$];
-    our $VERSION_BASE = 27; our $UNSTABLE_RELEASE = 4; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new(($VERSION_BASE))->numify / 1000), $UNSTABLE_RELEASE);
+    our $VERSION_BASE = 27; our $UNSTABLE_RELEASE = 5; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new(($VERSION_BASE))->numify / 1000), $UNSTABLE_RELEASE);
     our $PRODUCT_TOKEN = qq[Net::BitTorrent/$VERSION ($^O)];    # ext protocol
 
-    sub gen_peerid { #
+    sub gen_peerid {                                            #
         return pack(
             q[a20],
             (sprintf(
                  q[NB%03d%1s-%8s%5s],
-                 (q[$Rev: 28 $] =~ m[(\d+)]g),
+
+                 #(q[$Rev: 28 $] =~ m[(\d+)]g),
+                 $VERSION_BASE,
                  ($UNSTABLE_RELEASE ? q[S] : q[C]),
                  (join q[],
                   map {
@@ -32,7 +35,7 @@ package Net::BitTorrent::Version;
 
 =head1 NAME
 
-Net::BitTorrent::Version - The Net::BitTorrent project-wide version numbers
+Net::BitTorrent::Version - Net::BitTorrent's project-wide version numbers
 
 =head1 DESCRIPTION
 
@@ -47,7 +50,7 @@ and the Peer ID generator.
 This section describes and provides examples of the Peer ID format used
 by the current release of the C<Net::BitTorrent> module.
 
-=head3 Format
+=head2 Format
 
 This non-standard format was developed to be URL-safe, unique to the
 implementation, and "human parsable."
@@ -61,7 +64,7 @@ L<signature|/Signature>.  Consider this example:
 Here, C<NB004S> is the header, C<-rogzGB1v> is the mid-section, and
 C<--SVN> is the trailing signature.
 
-=head4 Header
+=head3 Header
 
 Two uppercase characters ('C<NB>') followed by three digits representing
 the SVN revision number with leading zeros, a single character
@@ -74,7 +77,7 @@ If the client is a CPAN build, the sixth byte is the capital letter
 other characters in the sixth byte are unsupported and may indicate a bad
 client.
 
-=head4 Mid-section
+=head3 Mid-section
 
 Following that are eight random characters in the following range:
 
@@ -89,13 +92,13 @@ announce.  On a popular tracker, every bit (and byte) helps.
 This section is inspired by the pseudo-base64 set used by SHADOW's
 BitTornado client.
 
-=head4 Signature
+=head3 Signature
 
 The final five characters may be random or static and should not be used
 in identifying the version of the client software.  Some early versions
 even have my name in this spot.  .:shrugs:.
 
-=head3 CPAN Version Numbers
+=head2 CPAN Version Numbers
 
 Stable CPAN releases will have the sixth byte set to 'C<C>' (capital
 letter 'c').
@@ -113,7 +116,7 @@ make a v1.0 release of Net::BitTorrent on CPAN.  The information in this
 document and the means of generating the module's version number will
 need to reflect that.
 
-=head3 Examples
+=head2 Examples
 
 =over
 
@@ -130,6 +133,8 @@ Improper Peer ID; the sixth bit is neither 'C' nor 'S'.  Possibly fake.
 
 Completely legal Peer ID generated by SVN/Dev r65.
 
+=back
+
 =head1 Author
 
 Sanko Robinson <sanko@cpan.org> - http://sankorobinson.com/
